//  Copyright (c) 2012 The Board of Trustees of The University of Alabama
//  All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without
//  modification, are permitted provided that the following conditions
//  are met:
//
//  1. Redistributions of source code must retain the above copyright
//  notice, this list of conditions and the following disclaimer.
//  2. Redistributions in binary form must reproduce the above copyright
//  notice, this list of conditions and the following disclaimer in the
//  documentation and/or other materials provided with the distribution.
//  3. Neither the name of the University nor the names of the contributors
//  may be used to endorse or promote products derived from this software
//  without specific prior written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
//  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
//  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
//  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
//  THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
//  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
//  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
//   SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
//  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
//  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
//  OF THE POSSIBILITY OF SUCH DAMAGE.

#import "MotionGestureRecognizer.h"

typedef enum {
    MotionSwipeGestureRecognizerDirectionRight = 1 << 0,
    MotionSwipeGestureRecognizerDirectionLeft  = 1 << 1,
    MotionSwipeGestureRecognizerDirectionUp    = 1 << 2,
    MotionSwipeGestureRecognizerDirectionDown  = 1 << 3,
    MotionSwipeGestureRecognizerDirectionIn  = 1 << 4,
    MotionSwipeGestureRecognizerDirectionOut  = 1 << 5
} MotionSwipeGestureRecognizerDirection;

@interface MotionSwipeGestureRecognizer : MotionGestureRecognizer {
    
    LeapVector *startVector;
    LeapVector *endVector;
    __strong LeapVector *previousVector;
    
    //0 for no thresh, 1 for whole mesh size. Defaults to 0.8
    float minimumSwipeThreshold;
    
    NSInteger accelerationCounter;
    NSInteger decelerationCounter;
    MotionSwipeGestureRecognizerDirection direction;
}

@property (nonatomic, assign) MotionSwipeGestureRecognizerDirection possibleDirections;
@property (nonatomic, assign, readonly) MotionSwipeGestureRecognizerDirection direction;

-(id)initWithTarget:(id)target selector:(SEL)sel;

-(void)setMinimumSwipeThreshold:(float)minimumSwipeThreshold;

-(void)resetValues;

@end
